@@ -1,9 +1,10 @@
 from .utils.device import empty_device_cache, preferred_dtype, resolve_default_device, resolve_device, resolve_torch_device
-from .steering import SteeringMethod, SteeringType
+# Note: SteeringMethod and SteeringType import temporarily disabled due to missing dependencies
+# from .steering import SteeringMethod, SteeringType
 
 __all__ = [
-    "SteeringMethod",
-    "SteeringType",
+    # "SteeringMethod",
+    # "SteeringType",
     "empty_device_cache",
     "preferred_dtype",
     "resolve_default_device",
