@@ -7,7 +7,8 @@ This module provides ground truth evaluation using the lm-eval-harness framework
 import logging
 from typing import Any, Dict
 
-from wisent.core.activations import ActivationAggregationStrategy, Activations
+from wisent.core.activations.core.atoms import ActivationAggregationStrategy
+from wisent.core.activations.activations import Activations
 from wisent.core.layer import Layer
 
 logger = logging.getLogger(__name__)
@@ -636,7 +637,7 @@ class LMEvalHarnessGroundTruth:
         try:
             import json
 
-            eval_methods_path = "wisent_guard/parameters/benchmarks/benchmark_evaluation_methods.json"
+            eval_methods_path = "wisent/parameters/benchmarks/benchmark_evaluation_methods.json"
             with open(eval_methods_path) as f:
                 benchmark_methods = json.load(f)
                 return benchmark_methods.get(task_name, "text-generation")
