@@ -95,6 +95,7 @@ class WisentModel:
         elif self.device == "cuda":
             load_kwargs["dtype"] = torch.float16
             load_kwargs["device_map"] = "auto"
+            load_kwargs["attn_implementation"] = "flash_attention_2"  # Uses flash-attn for 2-4x speedup
         else:
             load_kwargs["dtype"] = torch.float32
             load_kwargs["device_map"] = None
@@ -258,6 +259,7 @@ class WisentModel:
         self,
         message: list[ChatMessage],
         add_generation_prompt: bool = True,
+        enable_thinking: bool = True,
     ) -> dict[str, torch.Tensor]:
         """
         Encode a single input in chat format.
@@ -267,6 +269,8 @@ class WisentModel:
                 list of {'role': str, 'content': str} dicts (chat messages).
             add_generation_prompt:
                 If True, append the model's generation prompt at the end.
+            enable_thinking:
+                If False, disable thinking/reasoning mode (prevents <think> tags for supported models like Qwen).
 
         returns:
             dict with 'input_ids' and 'attention_mask' tensors.
@@ -279,10 +283,10 @@ class WisentModel:
             >>> wm._encode_one(msgs, add_generation_prompt=True)
             {"input_ids": tensor([[...]]), "attention_mask": tensor([[...]])}
         """
-      
+
         ids = self.tokenizer.apply_chat_template(
-            message, tokenize=True, add_generation_prompt=add_generation_prompt, return_tensors="pt"
-        )[0] 
+            message, tokenize=True, add_generation_prompt=add_generation_prompt, enable_thinking=enable_thinking, return_tensors="pt"
+        )[0]
         return {
             "input_ids": ids,
             "attention_mask": torch.ones_like(ids),
@@ -292,6 +296,7 @@ class WisentModel:
         self,
         inputs: list[list[ChatMessage]],
         add_generation_prompt: bool = True,
+        enable_thinking: bool = True,
     ) -> dict[str, torch.Tensor]:
         """
         Batch-encode a list of chat messages.
@@ -301,7 +306,9 @@ class WisentModel:
                 list of chat messages (each a list of {'role','content'} dicts).
             add_generation_prompt:
                 If True, append the model's generation prompt at the end of each.
-        
+            enable_thinking:
+                If False, disable thinking/reasoning mode (prevents <think> tags for supported models like Qwen).
+
         returns:
             dict with batched 'input_ids' and 'attention_mask' tensors.
 
@@ -316,21 +323,21 @@ class WisentModel:
             >>> wm._batch_encode([msgs1, msgs2], add_generation_prompt=True)
             {"input_ids": tensor([[...],[...]]), "attention_mask": tensor([[...],[...]])}
         """
-        
+
         singles = []
         for item in inputs:
-            singles.append(self._encode_one(item, add_generation_prompt=add_generation_prompt))
+            singles.append(self._encode_one(item, add_generation_prompt=add_generation_prompt, enable_thinking=enable_thinking))
 
         batch = self.tokenizer.pad(singles, padding=True, return_tensors="pt")
 
-        batch = {k: v.to(resolve_torch_device()) for k, v in batch.items()}
+        batch = {k: v.to(self.device) for k, v in batch.items()}
 
         return batch
 
     @torch.inference_mode()
     def generate(
         self,
-        inputs: list[list[ChatMessage]],
+        inputs: list[list[ChatMessage]] | str,
         max_new_tokens: int = 128,
         temperature: float = 0.7,
         top_p: float = 0.95,
@@ -338,6 +345,8 @@ class WisentModel:
         num_return_sequences: int = 1,
         use_steering: bool = False,
         steering_plan: SteeringPlan | None = None,
+        enable_thinking: bool = True,
+        prompt_is_formatted: bool = False,
         **gen_kwargs: Any,
     ) -> list[str]:
         """
@@ -345,7 +354,7 @@ class WisentModel:
 
         attributes:
             inputs:
-                list of chat messages (each a list of {'role','content'} dicts).
+                list of chat messages (each a list of {'role','content'} dicts) OR pre-formatted string.
             max_new_tokens:
                 max tokens to generate (beyond the prompt).
             temperature:
@@ -361,6 +370,10 @@ class WisentModel:
             steering_plan:
                 optional SteeringPlan to use for this call only (overrides internal plan).
                 If None, uses the internal plan.
+            enable_thinking:
+                If False, disable thinking/reasoning mode (prevents <think> tags for supported models like Qwen).
+            prompt_is_formatted:
+                If True, inputs is a pre-formatted string with chat template already applied.
             **gen_kwargs:
                 additional kwargs passed to 'model.generate()'.
 
@@ -439,7 +452,23 @@ class WisentModel:
         if use_steering:
             self.apply_steering(steering_plan)
 
-        batch = self._batch_encode(inputs, add_generation_prompt=True)
+        if prompt_is_formatted and isinstance(inputs, str):
+            # Direct tokenization of pre-formatted prompt
+            tokenizer_output = self.tokenizer(
+                inputs,
+                return_tensors="pt",
+                padding=False,  # Single prompt, no padding needed
+                truncation=True,  # Avoid errors on long inputs
+                max_length=self.tokenizer.model_max_length  # Use model's actual limit
+            )
+            # Move tensors to the correct device (same as _batch_encode does)
+            batch = {
+                "input_ids": tokenizer_output["input_ids"].to(self.device),
+                "attention_mask": tokenizer_output["attention_mask"].to(self.device)
+            }
+        else:
+            # Current behavior: apply chat template
+            batch = self._batch_encode(inputs, add_generation_prompt=True, enable_thinking=enable_thinking)
 
         gen_out = self.hf_model.generate(
             **batch,
@@ -472,6 +501,7 @@ class WisentModel:
         collect_topk: int = 5,
         use_steering: bool = False,
         steering_plan: SteeringPlan | None = None,
+        enable_thinking: bool = True,
         **gen_kwargs: Any,
     ) -> tuple[list[str], list[GenerationStats]]:
         """
@@ -486,7 +516,7 @@ class WisentModel:
             temperature:
                 sampling temperature (0 = greedy, 1 = default sampling).
             top_p:
-                nucleus sampling probability (0 = no filtering, 1 = full filtering).    
+                nucleus sampling probability (0 = no filtering, 1 = full filtering).
             do_sample:
                 if False, uses greedy decoding (top_k=1).
             num_return_sequences:
@@ -498,6 +528,8 @@ class WisentModel:
             steering_plan:
                 optional SteeringPlan to use for this call only (overrides internal plan).
                 If None, uses the internal plan.
+            enable_thinking:
+                If False, disable thinking/reasoning mode (prevents <think> tags for supported models like Qwen).
             **gen_kwargs:
                 additional kwargs passed to 'model.generate()'.
 
@@ -537,7 +569,7 @@ class WisentModel:
         if use_steering:
             self.apply_steering(steering_plan)
 
-        batch = self._batch_encode(inputs, add_generation_prompt=True)
+        batch = self._batch_encode(inputs, add_generation_prompt=True, enable_thinking=enable_thinking)
 
         out = self.hf_model.generate(
             **batch,
@@ -600,7 +632,7 @@ class WisentModel:
     @torch.inference_mode()
     def generate_stream(
         self,
-        inputs: list[list[ChatMessage]],
+        inputs: list[list[ChatMessage]] | str,
         max_new_tokens: int = 128,
         temperature: float = 0.7,
         top_p: float = 0.95,
@@ -609,6 +641,8 @@ class WisentModel:
         steering_plan: SteeringPlan | None = None,
         skip_prompt: bool = True,
         skip_special_tokens: bool = True,
+        enable_thinking: bool = True,
+        prompt_is_formatted: bool = False,
         **gen_kwargs: Any,
     ) -> Iterable[str]:
         """
@@ -617,7 +651,8 @@ class WisentModel:
 
         attributes:
             inputs:
-                list of chat messages (each a list of {'role','content'} dicts). Currently only one conversation is supported.
+                list of chat messages (each a list of {'role','content'} dicts) OR pre-formatted string.
+                Currently only one conversation is supported.
             max_new_tokens:
                 max tokens to generate (beyond the prompt).
             temperature:
@@ -635,6 +670,10 @@ class WisentModel:
                 if True, the yielded text excludes the input prompt.
             skip_special_tokens:
                 if True, special tokens are removed from the yielded text.
+            enable_thinking:
+                If False, disable thinking/reasoning mode (prevents <think> tags for supported models like Qwen).
+            prompt_is_formatted:
+                If True, inputs is a pre-formatted string with chat template already applied.
             **gen_kwargs:
                 additional kwargs passed to 'model.generate()'.
 
@@ -642,14 +681,30 @@ class WisentModel:
             generated text chunks (str), as they become available.
         """
 
-        if len(inputs) != 1:
-            raise ValueError(
-                f"generate_stream currently supports exactly one conversation at a time (got {len(inputs)})."
-            )
         if use_steering:
             self.apply_steering(steering_plan)
 
-        batch = self._batch_encode(inputs, add_generation_prompt=True)
+        if prompt_is_formatted and isinstance(inputs, str):
+            # Direct tokenization of pre-formatted prompt
+            tokenizer_output = self.tokenizer(
+                inputs,
+                return_tensors="pt",
+                padding=False,  # Single prompt, no padding needed
+                truncation=True,  # Avoid errors on long inputs
+                max_length=self.tokenizer.model_max_length  # Use model's actual limit
+            )
+            # Move tensors to the correct device (same as _batch_encode does)
+            batch = {
+                "input_ids": tokenizer_output["input_ids"].to(self.device),
+                "attention_mask": tokenizer_output["attention_mask"].to(self.device)
+            }
+        else:
+            # Current behavior: apply chat template
+            if not isinstance(inputs, list) or len(inputs) != 1:
+                raise ValueError(
+                    f"generate_stream currently supports exactly one conversation at a time (got {type(inputs)} with {len(inputs) if isinstance(inputs, list) else 'N/A'} items)."
+                )
+            batch = self._batch_encode(inputs, add_generation_prompt=True, enable_thinking=enable_thinking)
 
         streamer = TextIteratorStreamer(
             self.tokenizer,
